#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	using Newtonsoft.Json; 
	// Additional reference to package needed 
	// using Newtonsoft.Json.dll from install folder C:\Program Files (x86)\NinjaTrader 8\bin64 
	// If you want to target a specific reference it needs to be added to the bin/custom folder and referenced from there instead
	//    C:\Users\[YourUsername]\Documents\NinjaTrader 8\bin\custom
	//    // Download dll via: https://github.com/JamesNK/Newtonsoft.Json/releases
	
	using System.Net.Http;
	
	public class WTT_AppDataStream : Indicator
	{
		//... set the cycle.tools endpoints
		Uri apiBase= new Uri("https://api.cycle.tools/");
		string streamEndpoint = "/api/stream/SubmitStreamData";
		
		private HttpClient client;
		
		bool historicalDataLoaded=false;
		string streamname="";
		string btypes="M";
		
		List<double> closeList = new List<double>();
		List<string> datesList = new List<string>();
		
        public class StreamData
        {
            public string streamid { get; set; }
            public string symbolname { get; set; }
            public string messagetype { get; set; }
            public List<string> Dates { get; set; }
            public List<double> Values { get; set; }
        }
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Sync data between cycle tools and local chart.";
				Name										= "WTT App DataStream";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= true;
				DisplayInDataBox							= false;
				DrawOnPricePanel							= false;
				DrawHorizontalGridLines						= false;
				DrawVerticalGridLines						= false;
				PaintPriceMarkers							= false;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Overlay;
				IsSuspendedWhileInactive					= false;
				ApiKey					= string.Empty;
				maxBars					= 1500;
				Prefix					= @"NT";
			}
			else if (State == State.Configure)
			{
			}
			else if (State == State.DataLoaded)
			{
				//....set the correct symbol id
				switch(BarsPeriod.BarsPeriodType) {
					case BarsPeriodType.Minute:
						btypes="M";
						break;
						
					case BarsPeriodType.Day:
						btypes="D";
						break;
						
					case BarsPeriodType.Week:
						btypes="W";
						break;
				}	
				streamname=Prefix+"-"+Bars.Instrument.MasterInstrument.Name+"-"+BarsPeriod.Value.ToString()+btypes;
				
				client = new HttpClient();
				client.BaseAddress = apiBase;
			}
		}

		protected override void OnBarUpdate()
		{
			try {
				
				//... Build the historical dataset at initial first call, and POST historical stream
				if (Count - 2 == CurrentBar && !historicalDataLoaded) 
				{
					try {
						
					
						//... Get and prepare the date/close data array
			            int seriesCount = Close.Count - 1;
			            closeList.Clear();
						datesList.Clear();
					
			            for (int i = 0; i <= seriesCount; i++)
						{
			                closeList.Add(Close.GetValueAt(i));
							datesList.Add(Time.GetValueAt(i).ToString("yyyy-MM-ddTHH:mm:ss"));
						}
						
						//... just take last maxBars from data source
						var myCloseList = Enumerable.Reverse(closeList).Take(maxBars).Reverse().ToList();
						var myDatesList = Enumerable.Reverse(datesList).Take(maxBars).Reverse().ToList();
					
						var postHistoricalDataSet = new StreamData { 
							Values = myCloseList,
							Dates = myDatesList,
							streamid=streamname,
							symbolname=Instrument.FullName,
							messagetype="history"
						};
						
						//Call the WhenToTrade Cycle Tools API endpoint for streaming data
						//var postResult = Task.Run<bool>(async () => await PostStreamData(postHistoricalDataSet)).Result;
						//var postResult = PostStreamData(postHistoricalDataSet).Result;
						var postOk=PostStreamData(postHistoricalDataSet);
						
						if (postOk) historicalDataLoaded=true;
						
					} catch (Exception ex)
					{
					  Print("POST HISTORICAL API ERROR: "+ex.Message);
					} 
				}
				//... Build the UPSERT dataset when a bar has finished, and POST upsert stream
				else if (Count - 2 == CurrentBar && historicalDataLoaded) 
				{
					try {
						
						
						//... Get and prepare the date/close data array
						closeList.Clear();
						datesList.Clear();
					
			            closeList.Add(Close[0]);
						datesList.Add(Time[0].ToString("yyyy-MM-ddTHH:mm:ss"));
					
						closeList.Add(Close[1]);
						datesList.Add(Time[1].ToString("yyyy-MM-ddTHH:mm:ss"));
					
						var postUpsertDataSet = new StreamData { 
							Values = closeList,
							Dates = datesList,
							streamid=streamname,
							symbolname=Instrument.FullName,
							messagetype="upsert"
						};
						
						//Call the WhenToTrade Cycle Tools API endpoint for streaming data
						//var postResult = Task.Run<bool>(async () => await PostStreamData(postUpsertDataSet)).Result;
						var postOk=PostStreamData(postUpsertDataSet);
						if (!postOk) historicalDataLoaded=false;
						
					} catch (Exception ex)
					{
					  Print("POST UPSERT API ERROR: "+ex.Message);
					}


				}
			} catch (Exception ex)
			{
			  Print("ONBarUpdate API ERROR: "+ex.Message);
			}
		}

		//... helper function to post the stream object to the cycle tools API 
		private bool PostStreamData(StreamData postDataSet)
		{
			try {
				//Call the WhenToTrade Cycle Tools API endpoint for streaming data
//				using (var client = new HttpClient())
//	            {
//					//WhenToTrade Cycle Tools API URL
//					client.BaseAddress = apiBase;	

					//Load the close dataset for HTTP API POST call
	                var contents = JsonConvert.SerializeObject(postDataSet);
	                StringContent POSTcontent = new StringContent(contents, Encoding.UTF8, "application/json");

					//POST API CALL
					var response = client.PostAsync(streamEndpoint+"?api_Key="+ApiKey, POSTcontent).Result;
					
					//Check for network errors
					if( response.IsSuccessStatusCode)
				    {
				        //APIresultContent = response.Content.ReadAsStringAsync().Result;
						return true;
				    }
				    else
				    {
						Print("POST GENERIC API ERROR CODE: "+response.StatusCode.ToString());
						string msg ="";
						if ((int)response.StatusCode==401) // HttpStatusCode.Unauthorized
						{
							msg = "Not Authorized. Check API Key.";
						} else {
				        	msg = response.Content.ReadAsStringAsync().Result;
						}
				        Print("API ERROR: "+msg);
						Alert("wttAPIalert", Priority.High, "API ERROR: "+msg, NinjaTrader.Core.Globals.InstallDir+@"\sounds\Alert1.wav", 10, Brushes.Black, Brushes.Yellow); 
						return false;
				    }
//	            }
			} catch (Exception ex)
			{
			  Print("POST GENERIC API ERROR: "+ex.Message+"->"+ex.InnerException.Message);
			  return false;
			}
		}
		
		#region Properties
		//[NinjaScriptProperty]
		[Display(Name="Api Key", Description="Cycle Tools Api Key, from cycle.tools", Order=1, GroupName="WhenToTrade")]
		public string ApiKey
		{ get; set; }

		[NinjaScriptProperty]
		[Display(Name="Prefix", Description="Symbol prefix for datafeed id", Order=2, GroupName="WhenToTrade")]
		public string Prefix
		{ get; set; }
		
		[Range(500, 2000)]
		[NinjaScriptProperty]
		[Display(Name="Max bars", Description="Maximum bars", Order=2, GroupName="Parameters")]
		public int maxBars
		{ get; set; }
		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private WTT_AppDataStream[] cacheWTT_AppDataStream;
		public WTT_AppDataStream WTT_AppDataStream(string prefix, int maxBars)
		{
			return WTT_AppDataStream(Input, prefix, maxBars);
		}

		public WTT_AppDataStream WTT_AppDataStream(ISeries<double> input, string prefix, int maxBars)
		{
			if (cacheWTT_AppDataStream != null)
				for (int idx = 0; idx < cacheWTT_AppDataStream.Length; idx++)
					if (cacheWTT_AppDataStream[idx] != null && cacheWTT_AppDataStream[idx].Prefix == prefix && cacheWTT_AppDataStream[idx].maxBars == maxBars && cacheWTT_AppDataStream[idx].EqualsInput(input))
						return cacheWTT_AppDataStream[idx];
			return CacheIndicator<WTT_AppDataStream>(new WTT_AppDataStream(){ Prefix = prefix, maxBars = maxBars }, input, ref cacheWTT_AppDataStream);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.WTT_AppDataStream WTT_AppDataStream(string prefix, int maxBars)
		{
			return indicator.WTT_AppDataStream(Input, prefix, maxBars);
		}

		public Indicators.WTT_AppDataStream WTT_AppDataStream(ISeries<double> input , string prefix, int maxBars)
		{
			return indicator.WTT_AppDataStream(input, prefix, maxBars);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.WTT_AppDataStream WTT_AppDataStream(string prefix, int maxBars)
		{
			return indicator.WTT_AppDataStream(Input, prefix, maxBars);
		}

		public Indicators.WTT_AppDataStream WTT_AppDataStream(ISeries<double> input , string prefix, int maxBars)
		{
			return indicator.WTT_AppDataStream(input, prefix, maxBars);
		}
	}
}

#endregion
